; A demo for the m_chord_* opcodes
; For the relevant demo chord search for DEMO (all capital letters)
<CsoundSynthesizer>
<CsOptions>
-o dac -d --messagelevel=2
</CsOptions>
; ==============================================
<CsInstruments>

sr	=	48000
ksmps	=	1
nchnls	=	2
0dbfs	=	1

#include "chords.inc"

; Instrument playing a sawtooth at given MIDI note
; p4 = iMidiNote
instr PlayMidi
	iPitch = cpsmidinn(p4)

	aOut = vco2(.2, iPitch, 0)
	outs(aOut, aOut)
endin

; Instrument playing triangle at given pitches
; p4 = iPitch
instr PlayCps
	iPitch = p4

	aOut = vco2(.2, iPitch, 4, .5)
	outs(aOut, aOut)
endin

; m_chord_play_midi demo
instr ChordMidi
	kNotes[] init 4; Note array
	; Chord Name
	Sname init ""
	; Four root notes in an array
	kRoots[] = fillarray(60, 65, 67, 58)
	kRootIndex init 0

	; Two chord types
	kChords[] = fillarray(2, 16)
	kChordIndex init -1

	; Chord name DEMO 1
	if (changed2(kChordIndex) != 0) then
		Sname = m_chord_get_name(kChords[kChordIndex])
		printks("\nPlaying chord %d: %s", 1/kr, kChords[kChordIndex], Sname)
	endif

	; Trigger to change root notes
	kRootTrig = metro(1)
	if (kRootTrig == 1) then
		kRootIndex = wrap:k((kRootIndex + 1), 0, 4)
		if (kRootIndex == 1) then
			kChordIndex = wrap:k((kChordIndex + 1), 0, 2)
		endif
	endif

	; MIDI chord DEMO, no inversion, set out-of-range notes to -1
	kNotes = m_chord_play_midi(kChords[kChordIndex], kRoots[kRootIndex], 0, 2)

	; If notes changed play the PlayMidi instrument
	if (kRootTrig == 1) then ; when root note changes play notes
		kNoteIndex = 0
		while (kNoteIndex <4) do
			if (kNotes[kNoteIndex] != -1) then
				schedulek("PlayMidi", 0, .5, kNotes[kNoteIndex])
			endif
			kNoteIndex += 1
		od
	endif
endin

; m_chord_play_cps demo
instr ChordCps
	; Initiliase array for pitches
	kPitches[] init 4
	; Set up an array with root notes
	kRoots[] = fillarray(60, 65, 67, 72)
	kRootIndex init -1

	; Array with two chords
	kChords[] = fillarray(4, 28)
	kChordIndex init -1

	; Chord name DEMO 2
	if (changed(kChordIndex) != 0) then
		Sname = m_chord_get_name(kChords[kChordIndex])
		printks("\nPlaying chord %d: %s", 1/kr, kChords[kChordIndex], Sname)
	endif

	; A trigger to change roots and chords
	kTrig = metro(1)
	if (kTrig == 1) then ; change root notes and chords
		kRootIndex = wrap:k((kRootIndex + 1), 0, 4)
		if (kRootIndex == 0) then
			kChordIndex = wrap:k((kChordIndex + 1), 0, 2)
		endif
	endif

	; CPS chord DEMO
	kPitches = m_chord_play_cps(kChords[kChordIndex], kRoots[kRootIndex], 0, 2)

	; Play notes using PlayCps instrument
	if (kTrig == 1) then
		kNoteIndex = 0
		while (kNoteIndex < 4) do
			if (kPitches[kNoteIndex] != -1) then
				schedulek("PlayCps", 0, .9, kPitches[kNoteIndex])
			endif
			kNoteIndex += 1
		od
	endif
endin

; Chord inversion demo
instr ChordInvert
	; Initialise pitch array
	kPitches[] init 4

	; Set up tuning table (just intonation)
	iTune = ftgenonce(0, 0, 128, -51, 12, 2, 440, 69, 1.0, 21/20, 9/8, 7/6, 5/4, 4/3, 7/5, 3/2, 14/9, 5/3, 7/4, 15/8, 2.0)

	; Initialise chord inversion
	kInversion init -1

	; Set up two chords
	kChords[] = fillarray(8, 36)
	kChordIndex init -1

	; Chord name DEMO 3
	if (changed2(kChordIndex) != 0) then
		Sname = m_chord_get_name(kChords[kChordIndex])
		printks("\nPlaying chord %d: %s", 1/kr, kChords[kChordIndex], Sname)
	endif

	; Trigger to change inversion and chord
	kTrig = metro(1)
	if (kTrig == 1) then
		kInversion = wrap:k((kInversion + 1), 0, 4)
		if (kInversion == 0) then
			kChordIndex = wrap:k((kChordIndex + 1), 0, 2)
		endif
	endif

	; Chord inversion DEMO
	kPitches = m_chord_play_cps(kChords[kChordIndex], 60, kInversion, 2, iTune)

	; Set up four oscillators and play continuously
	kAmps[] init 4

	; Mute notes which are 0
	if (kTrig == 1) then
		kNoteIndex = 0
		while (kNoteIndex < 4) do
			if (kPitches[kNoteIndex] == -1) then
				kAmps[kNoteIndex] = 0
			else
				kAmps[kNoteIndex] = .2
			endif
			kNoteIndex += 1
		od
	endif

	; Play four oscillators
	aOsc1 = vco2(kAmps[0], kPitches[0], 2, .5)
	aOsc2 = vco2(kAmps[1], kPitches[1], 2, .5)
	aOsc3 = vco2(kAmps[2], kPitches[2], 2, .5)
	aOsc4 = vco2(kAmps[3], kPitches[3], 2, .5)

	; Sum oscillator outputs and play them
	aSum = sum(aOsc1, aOsc2, aOsc3, aOsc4)
	outs(aSum, aSum)
endin

</CsInstruments>
; ==============================================
<CsScore>
i"ChordMidi" 0 8
i"ChordCps" 8 8
i"ChordInvert" 16 8
e
</CsScore>
</CsoundSynthesizer>
