/*
m_chord_play_cps - Generate a tri or tetrachord (output frequencies)

DESCRIPTION
Output the pitches, frequencies in Hz, of a chord with three or four notes.
Chords predefined in a function table, they are chosen by index, 0-42 in
version 1.0. The pitches are output in an array. For trichords the last note
will be -1 and should be ignored.
The name of a chord corresponding to an index can be retrieved by using
m_chord_get_name .

SYNTAX
kPitches[] m_chord_play_cps kChordIndex, kRootNote, kInversion [, kLimit, \
	kTuningFt]

INITIALIZATION

PERFORMANCE
kPithces[] - four element output array holding the pitches of the chord. A value
	of -1 should be ignored.
kChordIndex - Index of the chord to play, corresponding names can be retrieved
	by using m_chord_get_name, internally limited to 0-number of chords -1
kRootNote - MIDI Root note of the chord, internally limited to 0-127
kInversion - Inversion of the chord, between 0 and 3, internally limited
	to that range
kLimit - What to do with notes out of the 0-127 range (default 0)
	0 = pass, notes will be output regardless
	1 = limit, limit notes to 0-127 range
	2 = invalidate, set out of range notes to -1
kTuningFt - A tuning function generate with GEN51 (default 0)

CREDITS
Author: Jeanette C.
*/

opcode m_chord_play_cps, k[], kkkOO
	kChordIndex, kRootNote, kInversion, kLimit, kTuningFt xin

	; initialise arrays
	kNotes[] init 4
	kPitches[] init 4

	; Set up a trigger for the cpstun opcode
	kTrig = changed(kNotes[0], kNotes[1], kNotes[2], kNotes[3])

	; Internal index into the arrays
	kNoteIndex init 0

	; Generate notes with m_chord_play_midi
	kNotes m_chord_play_midi kChordIndex, kRootNote, kInversion, kLimit

	; Now process pitches
	if (kTrig != 0) then; only process when something has changed
		kNoteIndex = 0
		if (kTuningFt != 0) then ; use cpstun for conversion
			while (kNoteIndex <4) do
				if (kNotes[kNoteIndex] == -1) then
					kPitches[kNoteIndex] = -1
				else ; a note to process
					kPitches[kNoteIndex] = tablekt(kNotes[kNoteIndex], kTuningFt, 0)
				endif
				kNoteIndex += 1
			od
		else ; use cpsmidinn to convert MIDI to cps
			while (kNoteIndex <4) do
				if (kNotes[kNoteIndex] == -1) then ; just copy
					kPitches[kNoteIndex] = -1
				else ; process MIDI note
					kPitches[kNoteIndex] = cpsmidinn(kNotes[kNoteIndex])
				endif
				kNoteIndex += 1
			od
		endif ; tuning procedures
	endif ; input notes changed
	
	; Output the frequencies
	xout(kPitches)
endop
